/*
 * pmcmd	Send one or more command to a portmaster.
 *		Fast standalone C version.
 *
 * Version:	@(#)pmcmd.c  1.20  30-Dec-1997  miquels@cistron.nl
 *
 * Usage:	pmcmd terminal server [passwd]
 *
 *		If the password is not given on the command line, it
 *		will be looked for in the environment variable PMPASSWD.
 *		If that is also not found, it will be read from standard input.
 *
 *		If standard input is a tty, prompting will be done.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <getopt.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>

char inbuf[65];
int ptr = 64;
char *more = "-- Press Return for More -- ";
int morelen;

char line[128];

/*
 *	Add a character to a line.
 */
void add_to_line(int c)
{
	static int lineptr = 0;

	if (c == '\r') return;
	if (c < 0) {
		lineptr = 0;
		return;
	}
	line[lineptr] = c;
	if (lineptr < 128) lineptr++;
	if (c == '\n') {
		write(1, line, lineptr);
		lineptr = 0;
	}
}

/*
 *	Expect a string.
 */
int expect(int fd, char *word, int echo)
{
	unsigned char c;
	int found = 0;
	int len, r;

	len = strlen(word);
	if (len > 64) len = 64;

	r = 0;

	while(!found) {
		if (read(fd, &c, 1) <= 0) {
			r = -1;
			break;
		}
		if (echo) add_to_line(c);
		memmove(inbuf, inbuf + 1, 63);
		inbuf[63] = c;
		if (ptr > 0) ptr--;
		if (!strcmp(inbuf + 64 - morelen, more)) {
			write(fd, "\r", 1);
			add_to_line(-1);
			continue;
		}
		found = !strcmp(inbuf + 64 - len, word);
	}

	return (r == 0) ? found : -1;
}



/*
 *	Fill out a sockaddr_in struct.
 */
int getaddr(char *host, int port, struct sockaddr_in *sin)
{
	struct hostent *h;

	if ((h = gethostbyname(host)) == NULL) {
		fprintf(stderr, "pmcmd: ");
		herror(host);
		return -1;
	}
	memset(sin, 0, sizeof(struct sockaddr_in));
	sin->sin_family = AF_INET;
	sin->sin_port   = htons(port);
	sin->sin_addr.s_addr = *((long *)h->h_addr);
	return 0;
}

int main(int argc, char **argv)
{
	int s, n;
	char buf[1024];
	char prompt[64];
	char pwdbuf[64];
	struct sockaddr_in sin;
	char *host, *pass;
	int port;
	int tty = isatty(0);

	if (argc != 2 && argc != 3) {
		fprintf(stderr, "Usage: %s hostname [password]\n", argv[0]);
		exit(1);
	}
	host = argv[1];
	port = 23;
	morelen = strlen(more);

	if (argc == 2) {
		if ((pass = getenv("PMPASSWD")) == NULL) {
			if (tty)
				pass = getpass("Password: ");
			else {
				fgets(pwdbuf, sizeof(pwdbuf), stdin);
				strtok(pwdbuf, "\r\n");
				pass = pwdbuf;
			}
		}
	} else
		pass = argv[2];

	/* Open client socket. */
	if ((s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
		perror("socket");
		exit(1);
	}
	if (getaddr(host, port, &sin) < 0) {
		exit(1);
	}

	/* Connect to server. */
	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		fprintf(stderr, "pmcmd: connect %s: %s\n",
			host, strerror(errno));
		exit(1);
	}

	expect(s, "ogin: ", 0);
	write(s, "!root\r", 6);
	expect(s, "assword: ", 0);
	sprintf(buf, "%s\r", pass);
	write(s, buf, strlen(buf));

	/*
	 *	Now collect the portmaster prompt.
	 */
	ptr = 64;
	expect(s, "> ", 1);
#if 0
	prompt[0] = '\n';
	strcpy(prompt + 1, inbuf + ptr);
#else
	strcpy(prompt, inbuf + ptr);
#endif

	/*
	 *	Send commands to the portmaster and print output.
	 */
	if (tty) write(1, "Command> ", 9);
	while(fgets(buf, 256, stdin) != NULL) {
		n = strlen(buf);
		if (n >= 2) {
			buf[n - 1] = '\r';
			write(s, buf, n);
			add_to_line(-1);
			if (expect(s, prompt, 1) < 0)
				break;
		}
		if (tty) write(1, "Command> ", 9);
	}

	return 0;
}
